@@ -1,7 +1,6 @@
 #![allow(clippy::float_cmp)]
 #![deny(clippy::if_not_else)]
 #![deny(clippy::needless_borrow)]
-#![deny(clippy::trivial_regex)]
 #![deny(clippy::unimplemented)]
 #![warn(missing_docs)]
 
@@ -67,10 +66,10 @@
 //! If you're targeting Node.js, we recommend you to use N-API over WebAssembly,
 //! because it's faster and less-limited than the WebAssembly-build.
 
-use parser::{parse, Query};
+use parser::parse_browserslist_query;
 use std::cmp::Ordering;
 #[cfg(target_arch = "wasm32")]
-pub use wasm::execute;
+pub use wasm::browserslist;
 pub use {error::Error, opts::Opts, queries::Distrib};
 
 #[cfg(not(target_arch = "wasm32"))]
@@ -109,39 +108,37 @@ where
     S: AsRef<str>,
     I: IntoIterator<Item = S>,
 {
-    let mut distribs = vec![];
+    let query = queries
+        .into_iter()
+        .enumerate()
+        .fold(String::new(), |mut s, (i, query)| {
+            if i > 0 {
+                s.push_str(", ");
+            }
+            s.push_str(query.as_ref());
+            s
+        });
 
-    for query in queries {
-        parse(query.as_ref()).try_fold(&mut distribs, |distribs, current| {
-            let query_string = match current {
-                Query::And(s) => s,
-                Query::Or(s) => s,
-            };
+    let mut distribs = parse_browserslist_query(&query)?
+        .1
+        .into_iter()
+        .enumerate()
+        .try_fold(vec![], |mut distribs, (i, current)| {
+            if i == 0 && current.negated {
+                return Err(Error::NotAtFirst(current.raw.to_string()));
+            }
 
-            let is_exclude = query_string.starts_with("not");
-            let query_string = if is_exclude {
-                &query_string[4..]
+            let mut dist = queries::query(current.atom, opts)?;
+            if current.negated {
+                distribs.retain(|distrib| !dist.contains(distrib));
+            } else if current.is_and {
+                distribs.retain(|distrib| dist.contains(distrib));
             } else {
-                query_string
-            };
-
-            let mut queries = queries::query(query_string, opts)?;
-            if is_exclude {
-                distribs.retain(|q| !queries.contains(q));
-            } else {
-                match current {
-                    Query::And(_) => {
-                        distribs.retain(|q| queries.contains(q));
-                    }
-                    Query::Or(_) => {
-                        distribs.append(&mut queries);
-                    }
-                }
+                distribs.append(&mut dist);
             }
 
             Ok::<_, Error>(distribs)
         })?;
-    }
 
     distribs.sort_by(|a, b| match a.name().cmp(b.name()) {
         Ordering::Equal => {
